if __name__ == '__main__':
    from __init__ import *
else:
    from . import *
from psplpyProject.psplpy.other_utils import is_sys, recursive_convert


def tests():
    print('test other utils')
    assert is_sys(is_sys.LINUX) is True
    assert is_sys(is_sys.WINDOWS) is False

    data = [1, (2, [3, 4])]
    assert recursive_convert(data, to=list) == [1, [2, [3, 4]]]
    assert recursive_convert(data, to=tuple) == (1, (2, (3, 4)))


if __name__ == '__main__':
    tests()



if __name__ == '__main__':
    from __init__ import *
else:
    from . import *
from psplpyProject.psplpy.file_utils import auto_rename


def tests():
    print('test file utils')
    cat_path = rc_dir / 'cat.png'
    renamed_cat_path = str(cat_path).replace('.png', '(1).png')
    print(renamed_cat_path)
    assert auto_rename(cat_path) == Path(renamed_cat_path)
    assert auto_rename(str(cat_path)) == renamed_cat_path


if __name__ == '__main__':
    tests()



import errno
import socket
import threading
import time
from pathlib import Path
from typing import Any, Callable


class ClientSocket:
    def __init__(self, host: str = '127.0.0.1', port: int = 12345, client_socket: socket.socket = None,
                 client_host: str = None, client_port: int = 12345):
        self.host = host
        self.port = port
        self.client_host = client_host
        self.client_port = client_port
        self.client_socket = client_socket

        self._length_bytes = 5

        if self.client_socket:
            self.client_socket.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, 32 * 1024 * 1024)

    def connect(self) -> None:
        self.client_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.client_socket.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, 32 * 1024 * 1024)
        # use the certain address of the client to connect the server
        if self.client_host and self.client_port:
            self.client_socket.bind((self.client_host, self.client_port))
        self.client_socket.connect((self.host, self.port))

    def _get_length(self, data: bytes) -> bytes:
        # for 5 bytes unsigned int, the max data length is 2**40 - 1, namely about 1tb
        bytes_result = len(data).to_bytes(self._length_bytes, byteorder='big')
        return bytes_result

    def _recv_length(self) -> int:
        byte_result = self.client_socket.recv(self._length_bytes)
        return int.from_bytes(byte_result, byteorder='big')

    def send(self, data: bytes):
        return self.client_socket.sendall(self._get_length(data) + data)

    def recv(self) -> bytes:
        length = self._recv_length()
        data = bytearray()
        while len(data) < length:
            data += self.client_socket.recv(length - len(data))
        return bytes(data)

    def recvf(self, output_path: str | Path, bufsize: int = 1024 * 1024 * 16) -> None:
        with open(output_path, 'wb') as f:
            while True:
                data = self.client_socket.recv(bufsize)
                if not data:
                    break
                f.write(data)

    def sendf(self, input_path: str | Path, bufsize: int = 1024 * 1024 * 16) -> None:
        with open(input_path, 'rb') as f:
            while True:
                data = f.read(bufsize)
                if not data:
                    break
                self.client_socket.send(data)

    def close(self) -> None:
        return self.client_socket.close()


class ServerSocket:
    def __init__(self, host: str = '127.0.0.1', port: int = 12345, backlog: int = 64):
        self.host = host
        self.port = port
        self.server_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.server_socket.bind((self.host, self.port))
        self.server_socket.listen(backlog)
        self.server_socket.setblocking(False)

    def accept(self) -> tuple[ClientSocket, Any] | tuple[None, None]:
        while True:
            try:
                client_socket, addr = self.server_socket.accept()
                return ClientSocket(client_socket=client_socket), addr
            except socket.error as e:
                if e.errno == errno.EWOULDBLOCK or e.errno == errno.EAGAIN:
                    time.sleep(0.01)
                    continue
                elif e.errno == errno.EBADF:
                    return None, None
                else:
                    raise e

    def handle(self, handler: Callable, *args, **kwargs) -> None:
        def _handle():
            while True:
                client_socket, addr = self.accept()
                if not (client_socket and addr):
                    break
                handle_thread = threading.Thread(target=handler, args=(client_socket, addr, *args), kwargs=kwargs)
                handle_thread.daemon = True
                handle_thread.start()
        threading.Thread(target=_handle).start()

    def close(self) -> None:
        self.server_socket.close()



import platform
from abc import ABC, abstractmethod
from typing import Any, Type


class FunctionClass(ABC):
    def __new__(cls, *args, **kwargs) -> Any:
        instance = super().__new__(cls)
        return instance(*args, **kwargs)

    @abstractmethod
    def __call__(self, *args, **kwargs): ...


class is_sys(FunctionClass):
    WINDOWS = 'Windows'
    LINUX = 'Linux'
    MACOS = 'Darwin'

    def __new__(cls, os_name: str) -> bool:
        return super().__new__(cls, os_name)

    def __call__(self, os_name: str) -> bool:
        this_os_name = platform.system()
        if os_name == this_os_name:
            return True
        return False


def recursive_convert(data: list | tuple, to: Type) -> tuple | list:
    """Recursively convert the lists and tuples are nested within each other to only tuples or lists"""
    if isinstance(data, (list, tuple)):
        return to(recursive_convert(item, to=to) for item in data)
    return data
    


if __name__ == '__main__':
    import test_middleware_utils
    import test_serialization_utils
    import test_concurrency_utils
    import test_network_utils
    import test_file_utils
    import test_image_utils
    import test_other_utils
    import test_dynamic_compose
else:
    from . import test_middleware_utils
    from . import test_serialization_utils
    from . import test_concurrency_utils
    from . import test_network_utils
    from . import test_file_utils
    from . import test_image_utils
    from . import test_other_utils
    from . import test_dynamic_compose


def global_test():
    test_middleware_utils.tests()
    test_serialization_utils.tests()
    test_concurrency_utils.tests()
    test_network_utils.tests()
    test_file_utils.tests()
    test_image_utils.tests()
    test_other_utils.tests()
    test_dynamic_compose.tests()


if __name__ == '__main__':
    global_test()



import threading
from typing import Any, Callable
import pika
from serialization_utils import Serializer


class Rabbitmq:
    HOST = 'localhost'
    PORT = 5672
    JSON = 'j'
    PICKLE = 'p'

    def __init__(self, host: str = None, port: int = None, serializer: str = JSON, compress: bool = False):
        self.host = host or self.HOST
        self.port = port or self.PORT
        self.serializer = serializer
        self.compress = compress

        self._serializer = Serializer()
        self.connection = pika.BlockingConnection(pika.ConnectionParameters(self.host, self.port))
        self.channel = self.connection.channel()

        self._stop_consuming = False
        self._suppress_error = False

    def _serialize(self, data: Any) -> bytes:
        if self.serializer == self.PICKLE:
            return self.PICKLE.encode() + self._serializer.dump_pickle(data, compress=self.compress)
        elif self.serializer == self.JSON:
            return self.JSON.encode() + self._serializer.dump_json(data, compress=self.compress)
        else:
            raise ValueError(f'Unsupported serializer: {self.serializer}')

    def _deserialize(self, data: bytes) -> Any:
        serializer = chr(data[0])
        data = data[1:]
        if serializer == self.JSON:
            return self._serializer.load_json(data, decompress=self.compress)
        elif serializer == self.PICKLE:
            return self._serializer.load_pickle(data, decompress=self.compress)
        else:
            raise ValueError(f'Unsupported serializer: {self.serializer}')

    @staticmethod
    def _default_callback(ch, method, properties, body) -> None:
        print(body)

    def _callback(self, ch, method, properties, body) -> None:
        body = self._deserialize(body)
        self.callback(ch, method, properties, body)
        if self._stop_consuming:
            ch.stop_consuming()

    def send_init(self, exchange: str, routing_keys: list[str]) -> None:
        self.exchange = exchange
        self.routing_keys = routing_keys
        self.channel.exchange_declare(exchange=self.exchange, exchange_type='topic')

    def recv_init(self, exchange: str, binding_keys: list[str], callback: Callable = None) -> None:
        self.send_init(exchange, routing_keys=binding_keys)
        self.callback = callback or self._default_callback
        queue = self.channel.queue_declare('', exclusive=True)
        self.queue_name = queue.method.queue
        for routing_key in self.routing_keys:
            self.channel.queue_bind(exchange=self.exchange, queue=self.queue_name, routing_key=routing_key)
        self.channel.basic_consume(queue=self.queue_name, on_message_callback=self._callback, auto_ack=True)

    def basic_publish(self, body: Any) -> None:
        for routing_key in self.routing_keys:
            self.channel.basic_publish(exchange=self.exchange, routing_key=routing_key, body=self._serialize(body))

    def _start_consuming(self) -> None:
        try:
            self.channel.start_consuming()
        except Exception as e:
            if not self._suppress_error:
                raise e

    def start_consuming(self) -> threading.Thread:
        t = threading.Thread(target=self._start_consuming)
        t.start()
        return t

    def stop_consuming(self):
        self._stop_consuming = True

    def close(self, suppress_error: bool = False) -> None:
        self._suppress_error = suppress_error
        try:
            self.connection.close()
        except Exception as e:
            if not self._suppress_error:
                raise e



if __name__ == '__main__':
    from __init__ import *
else:
    from . import *
from psplpyProject.psplpy.dynamic_compose import DynamicCompose

dc_rc_dir = rc_dir / 'dynamic_compose'
dc_compose_dir = dc_rc_dir / '.compose'
compose_dumped_file = dc_compose_dir / 'compose-dumped.yml'
dockerfile_dumped_file = dc_compose_dir / 'Dockerfile-dumped'


def tests():
    DynamicCompose.CWD = dc_rc_dir
    dc = DynamicCompose()
    dc.compose_dir = dc_compose_dir

    dc.format_compose()
    dc.format_dockerfile()
    dc.dump()

    assert compose_dumped_file.read_text().strip() == dc.compose_file.read_text().strip()
    # dc.compose_file.unlink()
    assert dockerfile_dumped_file.read_text().strip() == dc.dockerfile_file.read_text().strip()
    # dc.dockerfile_file.unlink()

    dc.up()


if __name__ == '__main__':
    tests()



import random

if __name__ == '__main__':
    from __init__ import *
else:
    from . import *
from psplpyProject.psplpy.serialization_utils import Serializer, Compressor


def tests():
    serializer = Serializer()
    serializer.compressor = Compressor(Compressor.ZLIB)

    lzma_serializer = Serializer()
    bench_data = {}
    rand_round = 10000
    for i in range(rand_round):
        bench_data[str(random.randint(0, rand_round))] = random.uniform(0, rand_round)
    p = PerfCounter()

    serialized_data = serializer.dump_pickle(bench_data)
    print(f'len: {len(serialized_data)},\t elapsed: {p.elapsed():.4f},\t dumps_pickle')
    serialized_data = serializer.dump_pickle(bench_data, to_b64=True)
    print(f'len: {len(serialized_data)},\t elapsed: {p.elapsed():.4f},\t dumps_pickle(to_b64)')
    serialized_data = serializer.dump_pickle(bench_data, compress=True)
    print(f'len: {len(serialized_data)},\t elapsed: {p.elapsed():.4f},\t dumps_pickle(zlib)')
    serialized_data = serializer.dump_pickle(bench_data, compress=True, to_b64=True)
    print(f'len: {len(serialized_data)},\t elapsed: {p.elapsed():.4f},\t dumps_pickle(zlib, to_b64)')
    serialized_data = lzma_serializer.dump_pickle(bench_data, compress=True)
    print(f'len: {len(serialized_data)},\t elapsed: {p.elapsed():.4f},\t dumps_pickle(lzma)')

    serialized_data = serializer.dump_json(bench_data)
    print(f'len: {len(serialized_data)},\t elapsed: {p.elapsed():.4f},\t dumps_json')
    serialized_data = serializer.dump_json(bench_data, compress=True)
    print(f'len: {len(serialized_data)},\t elapsed: {p.elapsed():.4f},\t dumps_json(zlib)')
    serialized_data = lzma_serializer.dump_json(bench_data, compress=True)
    print(f'len: {len(serialized_data)},\t elapsed: {p.elapsed():.4f},\t dumps_json(lzma)')

    serialized_data = serializer.dump_yaml(bench_data)
    print(f'len: {len(serialized_data)},\t elapsed: {p.elapsed():.4f},\t dumps_yaml')
    serialized_data = serializer.dump_yaml(bench_data, compress=True)
    print(f'len: {len(serialized_data)},\t elapsed: {p.elapsed():.4f},\t dumps_yaml(zlib)')
    serialized_data = lzma_serializer.dump_yaml(bench_data, compress=True)
    print(f'len: {len(serialized_data)},\t elapsed: {p.elapsed():.4f},\t dumps_yaml(lzma)')


    python_data = {1: '100', 2: 200, 3: ['你好', [3.14, None, False]]}

    dumps_data = serializer.dump_json(python_data, minimum=True)
    loads_data = serializer.load_json(dumps_data, trans_key_to_num=True)
    assert dumps_data == b'{"1":"100","2":200,"3":["\xe4\xbd\xa0\xe5\xa5\xbd",[3.14,null,false]]}', dumps_data
    assert loads_data == python_data

    dumps_data = serializer.dump_json(python_data, compress=True)
    loads_data = serializer.load_json(dumps_data, decompress=True, trans_key_to_num=True)
    assert dumps_data == (b"x\x9c\xab\xe6R\x00\x02%C%+ a`\xa0\xa4\x03\xe1\x1b\x01\xf9F\x06\x06P\x9e1\x90\x17\rf\x82"
                          b"\xb9O\xf6.x\xbat/T)\x08 \xe4@\xc0X\xcf\xd0D\x07E$\xaf4'\x07U$-1\xa78\x15.\x12\xcb\x05!k"
                          b"\x01E \x1ab"), dumps_data
    assert loads_data == python_data, loads_data

    dumps_data = serializer.dump_pickle(loads_data)
    loads_data = serializer.load_pickle(dumps_data)
    assert dumps_data == (b'\x80\x04\x95/\x00\x00\x00\x00\x00\x00\x00}\x94(K\x01\x8c\x03100\x94K\x02K\xc8K\x03]\x94('
                          b'\x8c\x06\xe4\xbd\xa0\xe5\xa5\xbd\x94]\x94(G@\t\x1e\xb8Q\xeb\x85\x1fN\x89eeu.'), loads_data
    assert loads_data == python_data

    dumps_data = serializer.dump_pickle(loads_data, compress=True, to_b64=True)
    loads_data = serializer.load_pickle(dumps_data, decompress=True, from_b64=True)
    assert dumps_data == b'eJxrYJmqzwABtVM0vBl7mA0NDKZ4M3mf8GaOnaLRw/Zk74KnS/dOAbLdHTjldgS+bpX360xNLdUDAOePE5o=', dumps_data
    assert loads_data == python_data, loads_data

    dumps_data = serializer.dump_yaml(python_data)
    loads_data = serializer.load_yaml(dumps_data)
    assert dumps_data == (b"1: '100'\n2: 200\n3:\n- \xe4\xbd\xa0\xe5\xa5\xbd\n"
                          b"- - 3.14\n  - null\n  - false\n"), dumps_data
    assert loads_data == python_data

    serializer = Serializer(path=tmp_file, data_type=dict)
    loads_data = serializer.load_json()
    assert loads_data == dict(), loads_data

    serializer.dump_json(python_data)
    loads_data = serializer.load_json(trans_key_to_num=True)
    assert loads_data == python_data, loads_data

    serializer.dump_yaml(python_data)
    loads_data = serializer.load_yaml()
    assert loads_data == python_data, loads_data

    serializer.dump_pickle(python_data)
    loads_data = serializer.load_pickle()
    assert loads_data == python_data, loads_data

    serializer = Serializer(path=tmp_file, embedded=True)
    serializer.dump_pickle(python_data, compress=True, to_b64=True)
    loads_data = serializer.load_pickle(decompress=True, from_b64=True)
    print(tmp_file.read_bytes().decode(errors='backslashreplace'))
    assert loads_data == python_data, loads_data

    serializer.dump_json(python_data)
    loads_data = serializer.load_json(trans_key_to_num=True)
    print(tmp_file.read_bytes().decode(errors='backslashreplace'))
    assert loads_data == python_data, loads_data

    serializer.dump_yaml(python_data)
    loads_data = serializer.load_yaml()
    print(tmp_file.read_bytes().decode(errors='backslashreplace'))
    assert loads_data == python_data, loads_data

    tmp_file.unlink()


if __name__ == '__main__':
    tests()



import functools
import re
from pathlib import Path
from typing import Callable


def _path_operation(func: Callable) -> Callable:
    @functools.wraps(func)
    def wrapper(path: str | Path) -> str | Path:
        original_type = type(path)
        if isinstance(path, str):
            path = Path(path)
        elif not isinstance(path, Path):
            raise ValueError(f"Path must be a string or Path object, not {original_type}")

        new_path = func(path)

        if original_type is str:
            return str(new_path)
        return new_path

    return wrapper


@_path_operation
def auto_rename(path: str | Path) -> str | Path:
    """If the file name has existed, add (n) after the file name and return, or return the original name."""
    n = 0
    while path.exists():
        n += 1
        stem = re.sub(r"\(\d+\)$", "", path.stem)
        path = path.with_name(f'{stem}({n}){path.suffix}')
    return path



import socket
import threading
import time

if __name__ == '__main__':
    from __init__ import *
else:
    from . import *
from psplpyProject.psplpy.network_utils import ClientSocket, ServerSocket


def get_ip_address():
    try:
        temp_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        temp_socket.connect(("114.114.114.114", 80))  # connect to a public ip address
        ip_address = temp_socket.getsockname()[0]  # get the host's ip
        temp_socket.close()
        return ip_address
    except socket.error:
        return None


port = 12351
client_port = 54333
data = b"Hello World" * 1024 * 32


def sender():
    def handler(client_socket: ClientSocket, addr):
        print(addr)
        assert addr == (get_ip_address(), client_port)

        received_data = client_socket.recv()
        assert received_data == data
        client_socket.send(data)
        recv_tmp_file = tmp_dir / 'recv_tmp.tmp'
        client_socket.recvf(recv_tmp_file)
        assert tmp_file.read_text() == recv_tmp_file.read_text()

        tmp_file.unlink()
        recv_tmp_file.unlink()
        client_socket.close()
        server.close()

    server = ServerSocket(port=port)
    server.handle(handler)


def recver():
    client = ClientSocket(port=port, client_host=get_ip_address(), client_port=client_port)
    client.connect()
    client.send(data)
    received_data = client.recv()
    assert received_data == data
    tmp_file.write_bytes(data)
    client.sendf(tmp_file)
    client.close()


def tests():
    threading.Thread(target=sender).start()
    threading.Thread(target=recver).start()


if __name__ == '__main__':
    tests()



import base64
import io
import json
import lzma
import pickle
import yaml
import re
import zlib
from pathlib import Path
from types import ModuleType
from typing import Any


class Compressor:
    LZMA = lzma
    ZLIB = zlib

    def __init__(self, lib: ModuleType = LZMA, encoding: str = 'utf-8'):
        self.lib = lib
        self.encoding = encoding

    def compress(self, data: bytes) -> bytes:
        compressed_data = self.lib.compress(data)
        return compressed_data

    def decompress(self, compressed_data: bytes) -> bytes:
        data = self.lib.decompress(compressed_data)
        return data


class _DataInFile:
    _MARK = b'"""'
    _UNIQUE_MARK = b'2qIgKf9iDsVrsTMS'
    _PRE_MARK = _MARK + _UNIQUE_MARK
    _POST_MARK = _UNIQUE_MARK + _MARK
    _WIN_ESP = b'\r\n'

    def find_start_and_end_index(self, data: bytes) -> tuple[int, int]:
        start_index = data.find(self._PRE_MARK)
        end_index = data.find(self._POST_MARK)
        if start_index != -1 and end_index != -1:
            start_index = start_index + len(self._PRE_MARK)

            pre_esp = data[start_index: start_index + len(self._WIN_ESP)]
            post_esp = data[end_index - len(self._WIN_ESP): end_index]
            if pre_esp == self._WIN_ESP:
                start_index += len(self._WIN_ESP)
            else:
                start_index += 1
            if post_esp == self._WIN_ESP:
                end_index -= len(self._WIN_ESP)
            else:
                end_index -= 1
            return start_index, end_index
        return -1, -1

    def get_data(self, data: bytes) -> bytes:
        start_index, end_index = self.find_start_and_end_index(data)
        if start_index != -1:
            return data[start_index: end_index]
        return b''

    def replace_data(self, original_data: bytes, data: bytes) -> bytes:
        start_index, end_index = self.find_start_and_end_index(original_data)
        if start_index != -1:
            replaced_data = original_data[:start_index] + data + original_data[end_index:]
        else:
            replaced_data = (original_data + b'\n' + self._PRE_MARK + b'\n' + data + b'\n' + self._POST_MARK + b'\n')
        return replaced_data


class Serializer:

    def __init__(self, path: str | Path = None, encoding: str = 'utf-8', data_type: type = None,
                 embedded: bool = False):
        self.path = path
        self.encoding = encoding
        self.data_type = data_type
        self.embedded = embedded
        if self.embedded:
            self._data_in_file = _DataInFile()
        self.compressor = Compressor()

    @staticmethod
    def _check_int_or_float(input_str: str) -> type:
        int_pattern = r'^[-+]?\d+$'
        float_pattern = r'^[-+]?\d+(\.\d+)?$'

        if re.match(int_pattern, input_str):
            return int
        elif re.match(float_pattern, input_str):
            return float
        else:
            return str

    def _convert_json_dict_key_to_number(self, data: Any) -> Any:
        if isinstance(data, dict):
            # if data type is dict, convert it
            converted_dict = {}
            for key, value in data.items():
                if type(key) == str:
                    trans_type = self._check_int_or_float(key)
                    key = trans_type(key)
                # process the values in dict, using recursion
                value = self._convert_json_dict_key_to_number(value)
                converted_dict[key] = value
            return converted_dict
        elif isinstance(data, (list, tuple, set)):
            # if date type is list, tuple or set, process it recursively
            converted_list = []
            for item in data:
                converted_item = self._convert_json_dict_key_to_number(item)
                converted_list.append(converted_item)
            return type(data)(converted_list)
        else:
            # if it's other type, don't process
            return data

    @staticmethod
    def _get_empty_data_structure(data_type: type | None) -> dict | list | tuple | set | None:
        if data_type is None:
            return None
        types = (dict, list, tuple, set)
        if data_type in types:
            return data_type()
        else:
            raise TypeError(f"Unsupported data type {data_type}")

    def _load(self, lib: ModuleType, data: str | bytes = None, decompress: bool = False, from_b64: bool = False) -> Any:
        if not data:
            try:
                data = Path(self.path).read_bytes()
            except FileNotFoundError:  # when file not found
                return self._get_empty_data_structure(self.data_type)
        if self.embedded:
            data = self._data_in_file.get_data(data)
            # embed pickle data in file need to transform to string to avoid potential errors
            if lib is pickle and not from_b64:
                data = base64.b64decode(data.decode())
        if from_b64:
            data = base64.b64decode(data.decode())
        if decompress:
            data = self.compressor.decompress(data)

        if lib in [json, yaml]:
            if isinstance(data, bytes):
                data = data.decode(self.encoding)
            if lib is json:
                try:
                    deserialized_data = json.loads(data)
                except json.decoder.JSONDecodeError:  # when file is empty
                    return self._get_empty_data_structure(self.data_type)
            elif lib is yaml:
                deserialized_data = yaml.safe_load(data)
        elif lib is pickle:
            try:
                deserialized_data = pickle.loads(data)
            except EOFError:  # when file is empty
                return self._get_empty_data_structure(self.data_type)
        else:
            raise AssertionError
        return deserialized_data

    def _postprocess_loaded_json(self, json_data: Any, trans_key_to_num: bool = False) -> Any:
        if trans_key_to_num:
            json_data = self._convert_json_dict_key_to_number(json_data)
        if json_data is None:  # when value is null
            return self._get_empty_data_structure(self.data_type)
        return json_data

    def load_yaml(self, data: bytes = None, decompress: bool = False) -> Any:
        return self._load(yaml, data=data, decompress=decompress)

    def load_json(self, data: str | bytes = None, decompress: bool = False, trans_key_to_num: bool = False) -> Any:
        json_data = self._load(json, data=data, decompress=decompress)
        return self._postprocess_loaded_json(json_data, trans_key_to_num)

    def load_pickle(self, data: bytes = None, decompress: bool = False, from_b64: bool = False) -> Any:
        return self._load(pickle, data=data, decompress=decompress, from_b64=from_b64)

    def _dump(self, lib: ModuleType, data: bytes, compress: bool = False, to_b64: bool = False) -> bytes:
        if compress:
            data = self.compressor.compress(data)
        if to_b64:
            data = base64.b64encode(data)
        if self.embedded:
            if lib is pickle and not to_b64:
                data = base64.b64encode(data)
            original_data = Path(self.path).read_bytes()
            data = self._data_in_file.replace_data(original_data, data)
        if self.path:
            Path(self.path).write_bytes(data)
        return data

    def _get_json_dumps_data(self, data: Any, indent: int, ensure_ascii: bool, minimum: bool) -> bytes:
        if minimum:
            kwargs = {"separators": (',', ':')}
        else:
            kwargs = {"indent": indent}
        kwargs['ensure_ascii'] = ensure_ascii
        return json.dumps(data, **kwargs).encode(self.encoding)

    def dump_yaml(self, data: Any, compress: bool = False, allow_unicode: bool = True) -> bytes:
        string_io = io.StringIO()
        yaml.dump(data, string_io, allow_unicode=allow_unicode)
        data = string_io.getvalue().encode(self.encoding)
        return self._dump(yaml, data, compress=compress)

    def dump_json(self, data: Any, compress: bool = False, indent: int = 4,
                  ensure_ascii: bool = False, minimum: bool = False) -> bytes:
        data = self._get_json_dumps_data(data, indent, ensure_ascii, minimum)
        return self._dump(json, data, compress=compress)

    def dump_pickle(self, data: Any, compress: bool = False, to_b64: bool = False) -> bytes:
        data = pickle.dumps(data)
        return self._dump(pickle, data, compress=compress, to_b64=to_b64)
